<?php

namespace Ecommerce\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\Libs\Service\Login as LoginService;
use Symfony\Component\DependencyInjection\Loader\XmlFileLoader;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\Config\FileLocator;

class LoginMobileController {

    protected $loginService;

    public function __construct(LoginService $loginService) {
        $this->loginService = $loginService;
    }

    private function buildData($conditions, $ip, $bHash, $product_id) {
        $email = $conditions[0]['value'];
        $password = $conditions[1]['value'];
        $requester_url = $conditions[2]['value'];
        $service = $conditions[3]['value'] ? 'true' : 'false';
        $host = (isset($conditions[3])) ? 'true' : 'false';
        $port = (isset($conditions[4])) ? 'true' : 'false';
        return '{"filter":[{"name":"EMAIL","operator":"=","value":"'.$email.'"},{"name":"PASSWORD","operator":"=","value":"'.$password.'"},{"name":"REQUESTER_URL","operator":"=","value":"'.$requester_url.'"},{"name":"PRODUCT_ID","operator":"=","value":"'.$product_id.'"},{"name":"HOST","operator":"=","value":'.$host.'},{"name":"PORT","operator":"=","value":'.$port.'},{"name":"SERVICE","operator":"=","value":"'.$service.'"},{"name":"IP","operator":"=","value":"'.$ip.'"},{"name":"BHASH","operator":"=","value":"'.$bHash.'"}],"page":1,"requestType":"FilterData","origin":{"containerName":"LOGIN","widgetName":"LOGIN"}}';
    }

    private function buildHeader() {
        $header = array();
        $header[] = 'Content-Type: application/json';
        $header[] = 'Accept: application/json, text/plain, */*';
        $header[] = 'Connection: keep-alive';
        return $header;
    }
    
    private function getUrlLogin() {
        $loginProperties = json_decode(file_get_contents(__DIR__.'/../../../vendor/teknisa/libraries/config/config.json'), true);
        return $loginUrl = $loginProperties['loginUrl'];
    }
    
    private function getParameters() {
        $instanceManager = \Zeedhi\Framework\DependencyInjection\InstanceManager::getInstance();
        $container = $instanceManager->getContainer();
		
		if (!$container->hasParameter('projectid')) {
		    throw new \Exception("Has no projectId parameter.");
		}
		
		return $container->getParameter('projectid');
		
    }
    
    public function authenticate($request, $response) {
        
        $conditions = $request->getFilterCriteria()->getConditions();
        $loginUrl = $this->getUrlLogin();
        
        try {    
            $ip = $this->loginService->getCurrentIp();
            $bHash = $this->loginService->getBHash();
            $product_id = $this->getParameters();   
            $data = $this->buildData($conditions, $ip, $bHash, $product_id);
            $header = $this->buildHeader();
            
            $sessionLogin = curl_init($loginUrl.'/lm_login');
            curl_setopt($sessionLogin, CURLOPT_HTTPHEADER, $header);
            curl_setopt($sessionLogin, CURLOPT_POST, true);
            curl_setopt($sessionLogin, CURLOPT_POSTFIELDS, $data);
            curl_setopt($sessionLogin, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($sessionLogin, CURLOPT_HEADER, 1);
            $result = curl_exec($sessionLogin);
            curl_close($sessionLogin);
            
            preg_match_all('/^Set-Cookie:\s*PHPSESSID=([^;]*)/mi', $result, $cookie);
            preg_match_all('/^({"error":.*)$/mi', $result, $dataset);
            
        
            if ($dataset[0]) {
                $dataset = (array) (json_decode($dataset[1][0]));
                throw new \Exception($dataset['error']);
            } else {
                preg_match_all('/^({"dataset":.*)$/mi', $result, $dataset);
                $dataset = (array) (json_decode($dataset[1][0]));
        
                if (!isset($dataset[0])) {
                    $dataSet = (array) ($dataset['dataset']->userData);
                    $response->addDataSet(new DataSet('operator', $dataSet));
                    $response->addDataSet(new DataSet('tokenLogin', array('tokenLogin' => $cookie[1][0])));
                    $response->addDataSet(new DataSet('bhash', array('bhash' => $bHash)));
                } else {
                    throw new \Exception('Error while trying to login.');
                }
            }
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    public function requestToken($request, $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $bHash = $conditions[0]['value'];
        $tokenLogin = $conditions[1]['value'];
        
        
        $loginUrl = $this->getUrlLogin();
        $data = '{"CLIENT_ID":"http://libraries.teknisa.com/", "BHASH":"'.$bHash.'"}';
        $header = $this->buildHeader($data);
        $header[] = 'COOKIE: PHPSESSID='.$tokenLogin;
        
        $sessionLogin = curl_init($loginUrl.'/lm_requestToken');
        curl_setopt($sessionLogin, CURLOPT_HTTPHEADER, $header);
        curl_setopt($sessionLogin, CURLOPT_POST, true);
        curl_setopt($sessionLogin, CURLOPT_COOKIESESSION, true);
        curl_setopt($sessionLogin, CURLOPT_POSTFIELDS, $data);
        curl_setopt($sessionLogin, CURLOPT_RETURNTRANSFER, 1);
        $result = curl_exec($sessionLogin);
        $result = (array) (json_decode($result));
        
        $dataSet = (array) ($result['dataset']->token);
        $response->addDataSet(new DataSet('token', $dataSet));
        
        curl_close($sessionLogin);
    }

    public function testeConnection() {}
}